/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|

  Horloge Nixie 12 heures implantée sur millivoltmètre Digitec.
  Utilise un STM32 Black Pill (F401).
  Électro-Bidouilleur, Janvier 2023. https://www.youtube.com/Électro-Bidouilleur
  Licence de droits: Attribution 4.0 International (CC BY 4.0)  Lisez https://creativecommons.org/licenses/by/4.0/deed.fr
*/

//DIRECTIVES AU COMPILATEUR
#include <STM32RTC.h>                       // Bibliothèque de soutien de la fonction RTC intégrée du STM32
#include <Wire.h>                           // Bibliothèque I2C
#define I2C_ADDR_PCF 0x20                   // L'adresse I2C de la puce PCF8575

// Instantiation de l'objet STM32RTC
STM32RTC& rtc = STM32RTC::getInstance();

//VARIABLES GLOBALES
unsigned int minutes_anciennes;
bool points_clignot;
unsigned int secondes;
unsigned int minutes;
unsigned int heures;
bool points = 0;
bool diz_heure;
byte unit_heure;
byte diz_minut;
byte unit_minut;
unsigned int word_nixie;
String commande_ajust = "";
char rx_caract;

// Fonction qui envoi 16 bits à l'esclave I2C (ici, le PCF8575)
void Send_I2C_16b(unsigned int word_to_send)
{
  Wire.begin(); // PB7, PB6               // Ouverture de la liaison avec l'esclave
  Wire.beginTransmission(I2C_ADDR_PCF);   // Début de la transmission
  Wire.write(word_to_send & 0xFF);        // Envoi de l'octet LSB
  Wire.write(word_to_send >> 8);          // Envoi de l'octet MSB
  Wire.endTransmission();                 // Fin de la transmission
  Wire.end();                             // Fermeture de la liaison
}

// Fonction d'initialisation à la manière Arduino IDE
void setup()
{
  Serial.begin(9600);                       // Ouverture du port série
  Wire.begin();                             // Broches PB7 (SDA) et PB6 (SCL) par défaut
  rtc.setClockSource(STM32RTC::LSE_CLOCK);  // Selection du résonateur 32KHz comme source de l'horloge RTC
  rtc.begin();                              // initialisation de l'horloge RTC
}

// Fonction de boucle perpétuelle à la manière Arduino IDE
void loop()
{
  // Lecture de l'heure du RTC
  heures = rtc.getHours();
  minutes = rtc.getMinutes();
  secondes = rtc.getSeconds();

  // Conversion de l'heure de format 24H en format 12H
  if (heures == 0) heures = 12;
  if (heures > 12) heures = heures - 12;

  // Extraction et conditionnement des chiffres de l'heure pour correspondre aux segments Nixie
  diz_heure = (heures / 10) % 10;
  unit_heure = 15 - (heures % 10);          // Le "15 -" correspond à l'offset à appliquer pour allumer le bon segment de Nixie
  diz_minut = 15 - ((minutes / 10) % 10);
  unit_minut = 15 - (minutes % 10);

  // Traitement du clignotement des points divisant minutes et heures
  if ((secondes % 2) == 0) points_clignot = true;
  else points_clignot = false;

  // Formattage et envoi de la valeur 16 bits encodant les sorties du mux PCF8575 pour afficher l'heure
  word_nixie = ((diz_heure << 13) + (points_clignot << 12) + (unit_heure << 8) + (diz_minut << 4) + unit_minut);
  Send_I2C_16b(word_nixie);                 // Envoi des bits

  // Envoi de l'heure formattée en 24H au port série
  Serial.printf("%02d:%02d:%02d\n", heures, minutes, secondes);

  // Réception d'une commande d'ajustement sur le port série
  while (Serial.available() > 0)                            // Caractère reçu?
  {
    rx_caract = Serial.read();                              // Oui, lire le caractère
    if (rx_caract == '\n')                                  // Est-ce une fin de ligne?
    {                                             
      heures = (commande_ajust.substring(0, 2)).toInt();    // Oui, Extraire les valeurs d'heure, minutes et secondes
      minutes = (commande_ajust.substring(2, 4)).toInt();   //            "
      secondes = (commande_ajust.substring(4, 6)).toInt();  
      commande_ajust = "";                                  // Effacer la commande reçue
      rtc.setHours(heures);                                 // Programmer la nouvelle heure sur le port série
      rtc.setMinutes(minutes);                              //            "
      rtc.setSeconds(secondes);
      Serial.printf("Ajustement: %02d:%02d:%02d\n", heures, minutes, secondes); // Envoyer la nouvelle heure sur le port série
    }
    else commande_ajust += rx_caract;                       // Pas une fin de ligne reçue, ajouter le caractère à la commande reçue
  }
  // Balayage des chiffres à chaque minute
  if (minutes != minutes_anciennes)                         // Les minutes ont changé?
  {
    minutes_anciennes = minutes;                            // Oui, sauvegarder la nouvelle valeur des minutes.
    for (int i = 15 ; i > 5 ; i--)                          // Boucle de création du balayage des chiffres.
    {
      word_nixie = ((1 << 13) + (1 << 12) + (i << 8) + (i << 4) + i);
      Send_I2C_16b(word_nixie);                             // Envoi des bits au PCF8575
      delay(100);                                           // Pause pour l'effet visuel
    }
  }
  delay(100);                                               // Traitement complet à toutes les 100 millisecondes.
}
